import {
  Component,
  OnInit,
  ViewChild,
  ChangeDetectorRef,
  AfterViewInit
} from '@angular/core';
import { trigger, transition, style, animate } from '@angular/animations';
import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';
import { FlatTreeControl } from '@angular/cdk/tree';
import { VirtualScrollTreeFlatDataSource } from '../datasource/virtual-scroll-tree-flat.data-source';
import { MatTreeFlattener, MatTree } from '@angular/material/tree';

import { ucapAnimations } from './animation';

export enum DeptType {
  // R: 기관Root
  Root = 'R',
  // D: 본부
  Department = 'D',
  // T: 부서
  Team = 'T'
}

interface DeptInfo {
  /** 부서SEQ */
  seq: number;
  /** 부서명 */
  name: string;
  /** 부서타입 */
  type: DeptType;
  /** 본부SEQ */
  rootSeq: number;
  /** 상위부서SEQ */
  parentSeq: number;
}
interface OrganizationNode {
  deptInfo: DeptInfo;
  name: string;
  children?: OrganizationNode[];
}

interface FlatNode {
  expandable: boolean;
  name: string;
  level: number;
  deptInfo: DeptInfo;
}
const deptInfoList: DeptInfo[] = [
  {
    seq: 1,
    name: '사장실',
    type: DeptType.Root,
    rootSeq: 0,
    parentSeq: 0
  },
  {
    seq: 2,
    name: '고문실',
    type: DeptType.Department,
    rootSeq: 1,
    parentSeq: 1
  },
  {
    seq: 3,
    name: '아키텍처솔루션팀',
    type: DeptType.Department,
    rootSeq: 1,
    parentSeq: 1
  },
  {
    seq: 4,
    name: '사장님 O/H',
    type: DeptType.Team,
    rootSeq: 1,
    parentSeq: 2
  },
  {
    seq: 5,
    name: 'DT 사업부',
    type: DeptType.Department,
    rootSeq: 1,
    parentSeq: 3
  },
  {
    seq: 6,
    name: '클라우드 사업부',
    type: DeptType.Department,
    rootSeq: 1,
    parentSeq: 3
  },
  {
    seq: 7,
    name: 'LF 팀',
    type: DeptType.Team,
    rootSeq: 2,
    parentSeq: 2
  }
];

@Component({
  selector: 'app-tree',
  templateUrl: './tree.component.html',
  styleUrls: ['./tree.component.scss'],
  animations: [
    ucapAnimations,
    trigger('romvoeAdd', [
      transition('remove <=> add', [
        style({
          transform: `rotate(45deg)`,
          opacity: 0
        }),
        animate('.2s 0s ease-out')
      ])
    ])
  ]
})
export class TreeComponent implements OnInit, AfterViewInit {
  @ViewChild('cvsvOrganization', { static: false })
  cvsvOrganization: CdkVirtualScrollViewport;
  treeControl: FlatTreeControl<FlatNode>;
  treeFlattener: MatTreeFlattener<OrganizationNode, FlatNode>;
  dataSource: VirtualScrollTreeFlatDataSource<OrganizationNode, FlatNode>;

  constructor() {
    this.treeControl = new FlatTreeControl<FlatNode>(
      (node) => node.level,
      (node) => node.expandable
    );
    this.treeFlattener = new MatTreeFlattener<OrganizationNode, FlatNode>(
      (node: OrganizationNode, level: number) => {
        return {
          expandable: !!node.children && node.children.length > 0,
          name: node.name,
          level,
          deptInfo: node.deptInfo
        };
      },
      (node) => node.level,
      (node) => node.expandable,
      (node) => node.children
    );

    this.dataSource = new VirtualScrollTreeFlatDataSource<
      OrganizationNode,
      FlatNode
    >(this.treeControl, this.treeFlattener);

    const nodeMap = new Map<number, OrganizationNode>();
    const rootNodeList: OrganizationNode[] = [];
    const remainChildNodeList: OrganizationNode[] = [];

    deptInfoList.forEach((deptInfo) => {
      const node: OrganizationNode = {
        deptInfo,
        name: deptInfo.name,
        children: []
      };
      if (nodeMap.has(deptInfo.seq)) {
        return;
      }
      nodeMap.set(deptInfo.seq, node);

      if (0 === deptInfo.parentSeq) {
        rootNodeList.push(node);
        return;
      }

      if (nodeMap.has(deptInfo.parentSeq)) {
        nodeMap.get(deptInfo.parentSeq).children.push(node);
      } else {
        remainChildNodeList.push(node);
      }
    });

    remainChildNodeList.forEach((node) => {
      if (nodeMap.has(node.deptInfo.parentSeq)) {
        nodeMap.get(node.deptInfo.parentSeq).children.push(node);
      }
    });
    this.dataSource.data = rootNodeList;
  }

  ngOnInit(): void {}

  ngAfterViewInit(): void {
    this.dataSource.cdkVirtualScrollViewport = this.cvsvOrganization;

    this.cvsvOrganization.scrollToOffset(100);
  }

  hasChild = (_: number, node: FlatNode) => node.expandable;

  onClickNode(node: OrganizationNode) {
    // this.selected.emit(node.deptInfo);
    console.log(node);
  }

  isLastNode(node: FlatNode, depth: number): boolean {
    const i = this.findNodeIndex(node, depth);

    if (-1 === i) {
      return false;
    }

    if (i === this.dataSource.expandedDataSubject.value.length - 1) {
      return true;
    }

    const n = this.dataSource.expandedDataSubject.value[i];

    for (
      let idx = i + 1;
      idx < this.dataSource.expandedDataSubject.value.length;
      idx++
    ) {
      const element = this.dataSource.expandedDataSubject.value[idx];
      if (n.level === element.level) {
        return false;
      }
      if (n.level > element.level) {
        return true;
      }
    }
    return true;
  }

  appendDivArray(level: number): number[] {
    if (0 === level) {
      return [];
    }

    return Array.from({ length: level }, (_, i: number) => i);
  }

  private findNodeIndex(node: FlatNode, depth: number): number {
    if (!node) {
      return -1;
    }
    const i = this.dataSource.expandedDataSubject.value.findIndex((n) => {
      return node.deptInfo.seq === n.deptInfo.seq;
    });

    if (0 === depth) {
      return i;
    }
    return this.findNodeIndex(this.findParentNode(i), depth - 1);
  }

  private findParentNode(index: number): FlatNode {
    const node = this.dataSource.expandedDataSubject.value[index];

    for (let idx = index - 1; idx >= 0; idx--) {
      const n = this.dataSource.expandedDataSubject.value[idx];
      if (n.level === node.level - 1) {
        return n;
      }
    }
    return undefined;
  }
}
